@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 '''A reliable mail-retriever toolkit.
 
 getmail is a reliable, modular, extensible mail retriever with support for
@@ -12,15 +11,18 @@ You should have received a copy of the license in the file COPYING.
 '''
 
 import sys
-if sys.hexversion < 0x2030300:
-    raise ImportError('getmail version 4 requires Python version 2.3.3'
-                      ' or later')
 
-__version__ = '5.13'
+__version__ = '6.7'
+
+__py_required__ = '2.7.18'
+__py_required_hex__ = 0x20712f0
+
+if sys.hexversion < __py_required_hex__:
+    raise ImportError('getmail version %s requires Python version %s '
+                      'or later'%(__version__,__py_required__))
 
 __all__ = [
     'baseclasses',
-    'compatibility',
     'constants',
     'destinations',
     'exceptions',
