@@ -1,9 +1,9 @@
-#!/usr/bin/env python2.3
 '''Base classes used elsewhere in the package.
 
 '''
 
 __all__ = [
+    'run_command',
     'ConfigurableBase',
     'ForkingBase',
     'ConfInstance',
@@ -11,6 +11,7 @@ __all__ = [
     'ConfBool',
     'ConfInt',
     'ConfTupleOfStrings',
+    'ConfTupleOfUnicode',
     'ConfTupleOfTupleOfStrings',
     'ConfPassword',
     'ConfDirectory',
@@ -24,11 +25,63 @@ import os
 import time
 import signal
 import types
-import sets
+import codecs
+from collections import namedtuple
+import tempfile
+import errno
+
+from argparse import Namespace
+import subprocess
 
 from getmailcore.exceptions import *
 import getmailcore.logging
-from getmailcore.utilities import eval_bool, expand_user_vars
+from getmailcore.utilities import *
+
+if sys.version_info.major > 2:
+    TemporaryFile23 = lambda: tempfile.TemporaryFile('bw+')
+else:
+    TemporaryFile23 = lambda: tempfile.TemporaryFile('w+')
+
+#######################################
+def run_command(command, args):
+    # Simple subprocess wrapper for running a command and fetching its exit 
+    # status and output/stderr.
+    if args is None:
+        args = []
+    if type(args) == tuple:
+        args = list(args)
+
+    # Programmer sanity checks
+    assert type(command) in (bytes, unicode), (
+        'command is %s (%s)' % (command, type(command))
+    )
+    assert type(args) == list, (
+        'args is %s (%s)' % (args, type(args))
+    )
+    for arg in args:
+        assert type(arg) in (bytes, unicode), 'arg is %s (%s)' % (arg, type(arg))
+
+    stdout = tempfile.TemporaryFile()
+    stderr = tempfile.TemporaryFile()
+
+    cmd = [command] + args
+
+    try:
+        p = subprocess.Popen(cmd, stdout=stdout, stderr=stderr)
+    except OSError as o:
+        if o.errno == errno.ENOENT:
+            # no such file, command not found
+            raise getmailConfigurationError('Program "%s" not found' % command)
+        #else:
+        raise
+
+    rc = p.wait()
+    stdout.seek(0)
+    stderr.seek(0)
+    if sys.version_info.major == 2:
+        return (rc, stdout.read().strip(), stderr.read().strip())
+    else:
+        return (rc, stdout.read().decode().strip(), stderr.read().decode().strip())
 
 #
 # Base classes
@@ -56,7 +109,7 @@ class ConfItem:
                 )
             # Use default.
             return self.default
-        if type(val) is not self.dtype and val != self.default:
+        if not isinstance(val,self.dtype) and val != self.default:
             # Got value, but not of expected type.  Try to convert.
             if self.securevalue:
                 self.log.debug('converting %s to type %s\n'
@@ -70,7 +123,7 @@ class ConfItem:
                     val = eval_bool(val)
                 else:
                     val = self.dtype(eval(val))
-            except (ValueError, SyntaxError, TypeError), o:
+            except (ValueError, SyntaxError, TypeError) as o:
                 raise getmailConfigurationError(
                     '%s: configuration value (%s) not of required type %s (%s)'
                     % (self.name, val, self.dtype, o)
@@ -79,7 +132,7 @@ class ConfItem:
 
 class ConfInstance(ConfItem):
     def __init__(self, name, default=None, required=True):
-        ConfItem.__init__(self, name, types.InstanceType, default=default,
+        ConfItem.__init__(self, name, object, default=default,
                           required=required)
 
 class ConfString(ConfItem):
@@ -107,13 +160,47 @@ class ConfTupleOfStrings(ConfString):
             if type(tup) != tuple:
                 raise ValueError('not a tuple')
             val = tup
-        except (ValueError, SyntaxError), o:
+        except (ValueError, SyntaxError) as o:
             raise getmailConfigurationError(
                 '%s: incorrect format (%s)' % (self.name, o)
             )
         result = [str(item) for item in val]
         return tuple(result)
 
+class ConfTupleOfUnicode(ConfString):
+    def __init__(self, name, default=None, required=True, allow_specials=()):
+        ConfString.__init__(self, name, default=default, required=required)
+        self.specials = allow_specials
+
+    def validate(self, configuration):
+        _locals = dict([(v, v) for v in self.specials])
+        val = ConfItem.validate(self, configuration)
+        try:
+            if not val:
+                val = '()'
+            tup = eval(val, {}, _locals)
+            if tup in self.specials:
+                val = [tup]
+            else:
+                if type(tup) != tuple:
+                    raise ValueError('not a tuple')
+                vals = []
+                for item in tup:
+                    try:
+                        item = item.encode()
+                        vals.append(codecs.decode(item,'ascii'))
+                    except:
+                        try:
+                            vals.append(codecs.decode(item,'utf-8'))
+                        except UnicodeError as o:
+                            raise ValueError('not ascii or utf-8: %s' % item)
+                val = vals
+        except (ValueError, SyntaxError) as o:
+            raise getmailConfigurationError(
+                '%s: incorrect format (%s)' % (self.name, o)
+            )
+        return tuple(val)
+
 class ConfTupleOfTupleOfStrings(ConfString):
     def __init__(self, name, default=None, required=True):
         ConfString.__init__(self, name, default=default, required=required)
@@ -127,7 +214,7 @@ class ConfTupleOfTupleOfStrings(ConfString):
             if type(tup) != tuple:
                 raise ValueError('not a tuple')
             val = tup
-        except (ValueError, SyntaxError), o:
+        except (ValueError, SyntaxError) as o:
             raise getmailConfigurationError(
                 '%s: incorrect format (%s)' % (self.name, o)
             )
@@ -209,7 +296,7 @@ class ConfMboxPath(ConfString):
             )
         fd = os.open(val, os.O_RDWR)
         status_old = os.fstat(fd)
-        f = os.fdopen(fd, 'r+b')
+        f = os.fdopen(fd, 'r+')
         # Check if it _is_ an mbox file.  mbox files must start with "From "
         # in their first line, or are 0-length files.
         f.seek(0, 0)
@@ -220,7 +307,7 @@ class ConfMboxPath(ConfString):
         # Reset atime and mtime
         try:
             os.utime(val, (status_old.st_atime, status_old.st_mtime))
-        except OSError, o:
+        except OSError as o:
             # Not root or owner; readers will not be able to reliably
             # detect new mail.  But you shouldn't be delivering to
             # other peoples' mboxes unless you're root, anyways.
@@ -249,7 +336,7 @@ class ConfigurableBase(object):
         self.log = getmailcore.logging.Logger()
         self.log.trace()
         self.conf = {}
-        allowed_params = sets.Set([item.name for item in self._confitems])
+        allowed_params = set([item.name for item in self._confitems])
         for (name, value) in args.items():
             if not name in allowed_params:
                 self.log.warning('Warning: ignoring unknown parameter "%s" '
@@ -272,8 +359,8 @@ class ConfigurableBase(object):
             # New class-based configuration item
             self.log.trace('checking %s\n' % item.name)
             self.conf[item.name] = item.validate(self.conf)
-        unknown_params = sets.ImmutableSet(self.conf.keys()).difference(
-            sets.ImmutableSet([item.name for item in self._confitems])
+        unknown_params = frozenset(self.conf.keys()).difference(
+            frozenset([item.name for item in self._confitems])
         )
         for param in sorted(list(unknown_params), key=str.lower):
             self.log.warning('Warning: ignoring unknown parameter "%s" '
@@ -284,9 +371,7 @@ class ConfigurableBase(object):
     def _confstring(self):
         self.log.trace()
         confstring = ''
-        names = self.conf.keys()
-        names.sort()
-        for name in names:
+        for name in list(sorted(self.conf.keys())):
             if name.lower() == 'configparser':
                 continue
             if confstring:
@@ -298,6 +383,7 @@ class ConfigurableBase(object):
         return confstring
 
 #######################################
+
 class ForkingBase(object):
     '''Base class for classes which fork children and wait for them to exit.
 
@@ -310,7 +396,7 @@ class ForkingBase(object):
         self.log.trace('handler called for signal %s' % sig)
         try:
             pid, r = os.wait()
-        except OSError, o:
+        except OSError as o:
             # No children on SIGCHLD.  Can't happen?
             self.log.warning('handler called, but no children (%s)' % o)
             return
@@ -354,14 +440,54 @@ class ForkingBase(object):
             raise getmailOperationError('child pid %d failed to exit'
                                         % self.__child_pid)
         exitcode = os.WEXITSTATUS(self.__child_status)
-
         return exitcode
 
+    def _pipemail(self, msg, delivered_to, received, unixfrom, stdout, stderr):
+        # Write out message
+        msgfile = TemporaryFile23()
+        msgfile.write(msg.flatten(delivered_to, received, include_from=unixfrom))
+        msgfile.flush()
+        os.fsync(msgfile.fileno())
+        # Rewind
+        msgfile.seek(0)
+        # Set stdin to read from this file
+        os.dup2(msgfile.fileno(), 0)
+        # Set stdout and stderr to write to files
+        os.dup2(stdout.fileno(), 1)
+        os.dup2(stderr.fileno(), 2)
+
+    def child_replace_me(self, msg, delivered_to, received, unixfrom,
+                         stdout, stderr, args, nolog=False):
+        self._pipemail(msg, delivered_to, received, unixfrom, stdout, stderr)
+        nolog or self.log.debug('about to execl() with args %s\n' % str(args))
+        os.execl(*args)
+
+    def forkchild(self, childfun, with_out=True):
+        self._prepare_child()
+        child = Namespace()
+        child.stdout = TemporaryFile23()
+        child.stderr = TemporaryFile23()
+        child.childpid = os.fork()
+        if child.childpid == 0:
+            childfun(child.stdout, child.stderr)
+        else:
+            self.log.debug('spawned child %d\n' % child.childpid)
+            child.exitcode = self._wait_for_child(child.childpid)
+            child.stderr.seek(0)
+            child.err = child.stderr.read().strip().decode()
+            child.stdout.seek(0)
+            if with_out:
+                child.out = child.stdout.read().strip()
+            return child
+
+    def get_msginfo(self, msg):
+        msginfo = {}
+        msginfo['sender'] = msg.sender.strip()
+        if msg.recipient != None:
+            rcpnt = msg.recipient.strip()
+            msginfo['recipient'] = rcpnt
+            msginfo['domain'] = rcpnt.lower().split('@')[-1]
+            msginfo['local'] = '@'.join(rcpnt.split('@')[:-1])
+        self.log.debug('msginfo "%s"\n' % msginfo)
+        return msginfo
 
-# For Python 2.3, which lacks the sorted() builtin
-if sys.hexversion < 0x02040000:
-    def sorted(l, key=lambda x: x):
-        lst = [(key(item), item) for item in l]
-        lst.sort()
-        return [val for (unused, val) in lst]
-    __all__.append('sorted')
