@@ -1,4 +1,3 @@
-#!/usr/bin/env python2.3
 '''Classes implementing destinations (files, directories, or programs getmail
 can deliver mail to).
 
@@ -24,11 +23,15 @@ __all__ = [
     'MultiSorter',
 ]
 
+import sys
 import os
 import re
-import tempfile
 import types
-import email.Utils
+
+try:
+    import email.Utils as Utils
+except ImportError:
+    import email.utils as Utils
 
 import pwd
 
@@ -36,6 +39,12 @@ from getmailcore.exceptions import *
 from getmailcore.utilities import *
 from getmailcore.baseclasses import *
 
+def exit_with(stderr,errmsg):
+    stderr.write(errmsg.encode())
+    stderr.flush()
+    os.fsync(stderr.fileno())
+    os._exit(127)
+
 #######################################
 class DeliverySkeleton(ConfigurableBase):
     '''Base class for implementing message-delivery classes.
@@ -74,13 +83,14 @@ class DeliverySkeleton(ConfigurableBase):
         ConfigurableBase.__init__(self, **args)
         try:
             self.initialize()
-        except KeyError, o:
+        except KeyError as o:
             raise getmailConfigurationError(
                 'missing required configuration parameter %s' % o
             )
         self.received_from = None
         self.received_with = None
         self.received_by = None
+        self.retriever = None
         self.log.trace('done\n')
 
     def retriever_info(self, retriever):
@@ -88,6 +98,7 @@ class DeliverySkeleton(ConfigurableBase):
         self.received_from = retriever.received_from
         self.received_with = retriever.received_with
         self.received_by = retriever.received_by
+        self.retriever = retriever
 
     def deliver_message(self, msg, delivered_to=True, received=True):
         self.log.trace()
@@ -96,6 +107,14 @@ class DeliverySkeleton(ConfigurableBase):
         msg.received_by = self.received_by
         return self._deliver_message(msg, delivered_to, received)
 
+    def some_security(self):
+        if ((os.geteuid() == 0 or os.getegid() == 0)
+                and not self.conf['allow_root_commands']):
+            raise getmailConfigurationError(
+                'refuse to invoke external commands as root '
+                'or GID 0 by default'
+            )
+
 #######################################
 class Maildir(DeliverySkeleton, ForkingBase):
     '''Maildir destination.
@@ -118,7 +137,7 @@ class Maildir(DeliverySkeleton, ForkingBase):
         self.dcount = 0
         try:
             self.conf['filemode'] = int(self.conf['filemode'], 8)
-        except ValueError, o:
+        except ValueError as o:
             raise getmailConfigurationError('filemode %s not valid: %s'
                                             % (self.conf['filemode'], o))
 
@@ -149,17 +168,12 @@ class Maildir(DeliverySkeleton, ForkingBase):
                 self.conf['path'], msg.flatten(delivered_to, received),
                 self.hostname, self.dcount, self.conf['filemode']
             )
-            stdout.write(f)
+            stdout.write(f.encode())
             stdout.flush()
             os.fsync(stdout.fileno())
             os._exit(0)
-        except StandardError, o:
-            # Child process; any error must cause us to exit nonzero for parent
-            # to detect it
-            stderr.write('maildir delivery process failed (%s)' % o)
-            stderr.flush()
-            os.fsync(stderr.fileno())
-            os._exit(127)
+        except Exception as o:
+            exit_with(stderr, 'maildir delivery process failed (%s)' % o)
 
     def _deliver_message(self, msg, delivered_to, received):
         self.log.trace()
@@ -179,34 +193,19 @@ class Maildir(DeliverySkeleton, ForkingBase):
                     raise getmailConfigurationError(
                         'refuse to deliver mail as GID 0'
                     )
-        self._prepare_child()
-        stdout = tempfile.TemporaryFile()
-        stderr = tempfile.TemporaryFile()
-        childpid = os.fork()
 
-        if not childpid:
-            # Child
-            self.__deliver_message_maildir(uid, gid, msg, delivered_to,
-                                           received, stdout, stderr)
-        self.log.debug('spawned child %d\n' % childpid)
-
-        # Parent
-        exitcode = self._wait_for_child(childpid)
-
-        stdout.seek(0)
-        stderr.seek(0)
-        out = stdout.read().strip()
-        err = stderr.read().strip()
+        child = self.forkchild(
+            lambda o,e: self.__deliver_message_maildir(
+                uid, gid, msg, delivered_to, received, o, e)
+            )
 
         self.log.debug('maildir delivery process %d exited %d\n'
-                       % (childpid, exitcode))
-
-        if exitcode or err:
+                       % (child.childpid, child.exitcode))
+        if child.exitcode or child.err:
             raise getmailDeliveryError('maildir delivery %d error (%d, %s)'
-                                       % (childpid, exitcode, err))
-
+                                       % (child.childpid, child.exitcode, child.err))
         self.dcount += 1
-        self.log.debug('maildir file %s' % out)
+        self.log.debug('maildir file %s' % child.out)
         return self
 
 #######################################
@@ -270,13 +269,13 @@ class Mboxrd(DeliverySkeleton, ForkingBase):
             # Open mbox file, refusing to create it if it doesn't exist
             fd = os.open(self.conf['path'], os.O_RDWR)
             status_old = os.fstat(fd)
-            f = os.fdopen(fd, 'r+b')
+            f = os.fdopen(fd, 'br+')
             lock_file(f, self.conf['locktype'])
             # Check if it _is_ an mbox file.  mbox files must start with "From "
             # in their first line, or are 0-length files.
             f.seek(0, 0)
             first_line = f.readline()
-            if first_line and not first_line.startswith('From '):
+            if first_line and not first_line.startswith(b'From '):
                 # Not an mbox file; abort here
                 unlock_file(f, self.conf['locktype'])
                 raise getmailDeliveryError('not an mboxrd file (%s)'
@@ -286,7 +285,7 @@ class Mboxrd(DeliverySkeleton, ForkingBase):
             try:
                 # Write out message plus blank line with native EOL
                 f.write(msg.flatten(delivered_to, received, include_from=True,
-                                    mangle_from=True) + os.linesep)
+                                    mangle_from=True) + os.linesep.encode())
                 f.flush()
                 os.fsync(fd)
                 status_new = os.fstat(fd)
@@ -294,17 +293,17 @@ class Mboxrd(DeliverySkeleton, ForkingBase):
                 try:
                     os.utime(self.conf['path'], (status_old.st_atime,
                              status_new.st_mtime))
-                except OSError, o:
+                except OSError as o:
                     # Not root or owner; readers will not be able to reliably
                     # detect new mail.  But you shouldn't be delivering to
                     # other peoples' mboxes unless you're root, anyways.
-                    stdout.write('failed to updated mtime/atime of mbox')
+                    stdout.write(b'failed to update mtime/atime of mbox')
                     stdout.flush()
                     os.fsync(stdout.fileno())
 
                 unlock_file(f, self.conf['locktype'])
 
-            except IOError, o:
+            except IOError as o:
                 try:
                     if not f.closed:
                         # If the file was opened and we know how long it was,
@@ -314,7 +313,7 @@ class Mboxrd(DeliverySkeleton, ForkingBase):
                         f.truncate(status_old.st_size)
                 except KeyboardInterrupt:
                     raise
-                except StandardError:
+                except Exception:
                     pass
                 raise getmailDeliveryError(
                     'failure writing message to mbox file "%s" (%s)'
@@ -323,13 +322,8 @@ class Mboxrd(DeliverySkeleton, ForkingBase):
 
             os._exit(0)
 
-        except StandardError, o:
-            # Child process; any error must cause us to exit nonzero for parent
-            # to detect it
-            stderr.write('mbox delivery process failed (%s)' % o)
-            stderr.flush()
-            os.fsync(stderr.fileno())
-            os._exit(127)
+        except Exception as o:
+            exit_with(stderr, 'mbox delivery process failed (%s)' % o)
 
     def _deliver_message(self, msg, delivered_to, received):
         self.log.trace()
@@ -351,37 +345,24 @@ class Mboxrd(DeliverySkeleton, ForkingBase):
                 raise getmailConfigurationError(
                     'refuse to deliver mail as GID 0'
                 )
-        self._prepare_child()
-        stdout = tempfile.TemporaryFile()
-        stderr = tempfile.TemporaryFile()
-        childpid = os.fork()
 
-        if not childpid:
-            # Child
-            self.__deliver_message_mbox(uid, gid, msg, delivered_to, received,
-                                        stdout, stderr)
-        self.log.debug('spawned child %d\n' % childpid)
-
-        # Parent
-        exitcode = self._wait_for_child(childpid)
-
-        stdout.seek(0)
-        stderr.seek(0)
-        out = stdout.read().strip()
-        err = stderr.read().strip()
+        child = self.forkchild(
+            lambda o,e: self.__deliver_message_mbox(
+                uid, gid, msg, delivered_to, received, o, e)
+            )
 
         self.log.debug('mboxrd delivery process %d exited %d\n'
-                       % (childpid, exitcode))
-
-        if exitcode or err:
+                       % (child.childpid, child.exitcode))
+        if child.exitcode or child.err:
             raise getmailDeliveryError('mboxrd delivery %d error (%d, %s)'
-                                       % (childpid, exitcode, err))
+                                       % (child.childpid, child.exitcode, child.err))
 
-        if out:
-            self.log.debug('mbox delivery: %s' % out)
+        if child.out:
+            self.log.debug('mbox delivery: %s' % child.out)
 
         return self
 
+
 #######################################
 class MDA_qmaillocal(DeliverySkeleton, ForkingBase):
     '''qmail-local MDA destination.
@@ -482,53 +463,30 @@ class MDA_qmaillocal(DeliverySkeleton, ForkingBase):
                 self.conf['localdomain'], msginfo['sender'],
                 self.conf['defaultdelivery']
             )
-            self.log.debug('about to execl() with args %s\n' % str(args))
             # Modify message
             if self.conf['strip_delivered_to']:
                 msg.remove_header('delivered-to')
                 # Also don't insert a Delivered-To: header.
                 delivered_to = None
-            # Write out message
-            msgfile = tempfile.TemporaryFile()
-            msgfile.write(msg.flatten(delivered_to, received))
-            msgfile.flush()
-            os.fsync(msgfile.fileno())
-            # Rewind
-            msgfile.seek(0)
-            # Set stdin to read from this file
-            os.dup2(msgfile.fileno(), 0)
-            # Set stdout and stderr to write to files
-            os.dup2(stdout.fileno(), 1)
-            os.dup2(stderr.fileno(), 2)
-            change_usergroup(self.log, self.conf['user'], self.conf['group'])
-            # At least some security...
-            if ((os.geteuid() == 0 or os.getegid() == 0)
-                    and not self.conf['allow_root_commands']):
-                raise getmailConfigurationError(
-                    'refuse to invoke external commands as root '
-                    'or GID 0 by default'
-                )
 
-            os.execl(*args)
-        except StandardError, o:
-            # Child process; any error must cause us to exit nonzero for parent
-            # to detect it
-            stderr.write('exec of qmail-local failed (%s)' % o)
-            stderr.flush()
-            os.fsync(stderr.fileno())
-            os._exit(127)
+            change_usergroup(self.log, self.conf['user'], self.conf['group'])
+            self.some_security()
+            self.child_replace_me(msg,
+                            delivered_to, received, False,
+                            stdout, stderr, args)
+        except Exception as o:
+            exit_with(stderr,'exec of qmail-local failed (%s)' % o)
 
     def _deliver_message(self, msg, delivered_to, received):
         self.log.trace()
-        self._prepare_child()
         if msg.recipient == None:
             raise getmailConfigurationError(
                 'MDA_qmaillocal destination requires a message source that '
                 'preserves the message envelope'
             )
         msginfo = {
-            'sender' : msg.sender,
-            'local' : '@'.join(msg.recipient.lower().split('@')[:-1])
+            'sender' : msg.sender.strip(),
+            'local' : '@'.join(msg.recipient.strip().lower().split('@')[:-1])
         }
 
         self.log.debug('recipient: extracted local-part "%s"\n'
@@ -555,37 +513,25 @@ class MDA_qmaillocal(DeliverySkeleton, ForkingBase):
         self.log.debug('recipient: set dash to "%s", ext to "%s"\n'
                        % (msginfo['dash'], msginfo['ext']))
 
-        stdout = tempfile.TemporaryFile()
-        stderr = tempfile.TemporaryFile()
-        childpid = os.fork()
+        child = self.forkchild(
+            lambda o,e: self._deliver_qmaillocal(
+                msg, msginfo, delivered_to, received, o, e)
+            )
 
-        if not childpid:
-            # Child
-            self._deliver_qmaillocal(msg, msginfo, delivered_to, received,
-                                     stdout, stderr)
-        self.log.debug('spawned child %d\n' % childpid)
 
-        # Parent
-        exitcode = self._wait_for_child(childpid)
+        self.log.debug('qmail-local %d exited %d\n' % (child.childpid, child.exitcode))
 
-        stdout.seek(0)
-        stderr.seek(0)
-        out = stdout.read().strip()
-        err = stderr.read().strip()
-
-        self.log.debug('qmail-local %d exited %d\n' % (childpid, exitcode))
-
-        if exitcode == 111:
+        if child.exitcode == 111:
             raise getmailDeliveryError('qmail-local %d temporary error (%s)'
-                                       % (childpid, err))
-        elif exitcode:
+                                       % (child.childpid, child.err))
+        elif child.exitcode:
             raise getmailDeliveryError('qmail-local %d error (%d, %s)'
-                                       % (childpid, exitcode, err))
+                                       % (child.childpid, child.exitcode, child.err))
 
-        if out and err:
-            info = '%s:%s' % (out, err)
+        if child.out and child.err:
+            info = '%s:%s' % (child.out, child.err)
         else:
-            info = out or err
+            info = child.out or child.err
 
         return 'MDA_qmaillocal (%s)' % info
 
@@ -608,6 +554,9 @@ class MDA_external(DeliverySkeleton, ForkingBase):
                     %(recipient) - recipient address
                     %(domain) - domain-part of recipient address
                     %(local) - local-part of recipient address
+                    %(mailbox) - for IMAP retrievers, the name of the
+                        server-side mailbox/folder the message was retrieved
+                        from.  Will be empty for POP.
 
                   Warning: the text of these replacements is taken from the
                   message and is therefore under the control of a potential
@@ -669,86 +618,51 @@ class MDA_external(DeliverySkeleton, ForkingBase):
     def _deliver_command(self, msg, msginfo, delivered_to, received,
                          stdout, stderr):
         try:
-            # Write out message with native EOL convention
-            msgfile = tempfile.TemporaryFile()
-            msgfile.write(msg.flatten(delivered_to, received,
-                                      include_from=self.conf['unixfrom']))
-            msgfile.flush()
-            os.fsync(msgfile.fileno())
-            # Rewind
-            msgfile.seek(0)
-            # Set stdin to read from this file
-            os.dup2(msgfile.fileno(), 0)
-            # Set stdout and stderr to write to files
-            os.dup2(stdout.fileno(), 1)
-            os.dup2(stderr.fileno(), 2)
-            change_usergroup(self.log, self.conf['user'], self.conf['group'])
-            # At least some security...
-            if ((os.geteuid() == 0 or os.getegid() == 0)
-                    and not self.conf['allow_root_commands']):
-                raise getmailConfigurationError(
-                    'refuse to invoke external commands as root '
-                    'or GID 0 by default'
-                )
             args = [self.conf['path'], self.conf['path']]
+            msginfo['mailbox'] = self.retriever.mailbox_selected or ''
+            if sys.version_info.major == 2:
+                msginfo['mailbox'] = msginfo['mailbox'].encode('utf-8')
             for arg in self.conf['arguments']:
                 arg = expand_user_vars(arg)
                 for (key, value) in msginfo.items():
                     arg = arg.replace('%%(%s)' % key, value)
                 args.append(arg)
-            self.log.debug('about to execl() with args %s\n' % str(args))
-            os.execl(*args)
-        except StandardError, o:
-            # Child process; any error must cause us to exit nonzero for parent
-            # to detect it
-            stderr.write('exec of command %s failed (%s)'
+
+            change_usergroup(self.log, self.conf['user'], self.conf['group'])
+            self.some_security()
+            self.child_replace_me(msg,
+                            delivered_to, received, self.conf['unixfrom'],
+                            stdout, stderr, args)
+        except Exception as o:
+            exit_with(stderr, 'exec of command %s failed (%s)'
                          % (self.conf['command'], o))
-            stderr.flush()
-            os.fsync(stderr.fileno())
-            os._exit(127)
 
     def _deliver_message(self, msg, delivered_to, received):
         self.log.trace()
-        self._prepare_child()
-        msginfo = {}
-        msginfo['sender'] = msg.sender
-        if msg.recipient != None:
-            msginfo['recipient'] = msg.recipient
-            msginfo['domain'] = msg.recipient.lower().split('@')[-1]
-            msginfo['local'] = '@'.join(msg.recipient.split('@')[:-1])
-        self.log.debug('msginfo "%s"\n' % msginfo)
+        msginfo = self.get_msginfo(msg)
 
-        stdout = tempfile.TemporaryFile()
-        stderr = tempfile.TemporaryFile()
-        childpid = os.fork()
-
-        if not childpid:
-            # Child
-            self._deliver_command(msg, msginfo, delivered_to, received,
-                                  stdout, stderr)
-        self.log.debug('spawned child %d\n' % childpid)
-
-        # Parent
-        exitcode = self._wait_for_child(childpid)
-
-        stdout.seek(0)
-        stderr.seek(0)
-        out = stdout.read().strip()
-        err = stderr.read().strip()
+        child = self.forkchild(
+            lambda o,e: self._deliver_command(
+                msg, msginfo, delivered_to, received, o, e)
+            )
 
         self.log.debug('command %s %d exited %d\n'
-                       % (self.conf['command'], childpid, exitcode))
-
-        if exitcode or (err and not self.conf['ignore_stderr']):
+                       % (self.conf['command'], child.childpid, child.exitcode))
+        if child.exitcode:
             raise getmailDeliveryError(
                 'command %s %d error (%d, %s)'
-                % (self.conf['command'], childpid, exitcode, err)
+                % (self.conf['command'], child.childpid, child.exitcode, child.err)
             )
-        elif err:
+        elif child.err:
+            if not self.conf['ignore_stderr']:
+                raise getmailDeliveryError(
+                    'command %s %d wrote to stderr: %s'
+                    % (self.conf['command'], child.childpid, child.err)
+                )
+            #else:
             # User said to ignore stderr, just log it.
-            self.log.info('command %s: %s' % (self, err))
-
-        return 'MDA_external command %s (%s)' % (self.conf['command'], out)
+            self.log.info('command %s: %s' % (self, child.err))
+        return 'MDA_external command %s (%s)' % (self.conf['command'], child.out)
 
 #######################################
 class MultiDestinationBase(DeliverySkeleton):
@@ -844,7 +758,7 @@ class MultiDestination(MultiDestinationBase):
         for item in dests:
             try:
                 dest = self._get_destination(item)
-            except getmailConfigurationError, o:
+            except getmailConfigurationError as o:
                 raise getmailConfigurationError('%s destination error %s'
                                                 % (item, o))
             self._destinations.append(dest)
@@ -901,13 +815,13 @@ class MultiSorterBase(MultiDestinationBase):
             for (pattern, path) in _locals:
                 try:
                     dest = self._get_destination(path)
-                except getmailConfigurationError, o:
+                except getmailConfigurationError as o:
                     raise getmailConfigurationError(
                         'pattern %s destination error %s' % (pattern, o)
                     )
                 self.targets.append((re.compile(pattern, re.IGNORECASE), dest))
                 self._destinations.append(dest)
-        except re.error, o:
+        except re.error as o:
             raise getmailConfigurationError('invalid regular expression %s' % o)
 
     def _confstring(self):
@@ -1043,7 +957,7 @@ class MultiGuesser(MultiSorterBase):
                     'looking for addresses in %s header fields\n' % field
                 )
                 header_addrs.extend(
-                    [addr for (name, addr) in email.Utils.getaddresses(
+                    [addr for (name, addr) in Utils.getaddresses(
                         msg.get_all(field, [])
                      ) if addr]
                 )
