@@ -1,4 +1,3 @@
-#!/usr/bin/env python2.3
 '''Classes implementing message filters.
 
 Currently implemented:
@@ -17,11 +16,11 @@ import tempfile
 import types
 
 from getmailcore.exceptions import *
-from getmailcore.compatibility import *
 from getmailcore.message import *
 from getmailcore.utilities import *
 from getmailcore.baseclasses import *
 
+
 #######################################
 class FilterSkeleton(ConfigurableBase):
     '''Base class for implementing message-filtering classes.
@@ -63,7 +62,7 @@ class FilterSkeleton(ConfigurableBase):
         ConfigurableBase.__init__(self, **args)
         try:
             self.initialize()
-        except KeyError, o:
+        except KeyError as o:
             raise getmailConfigurationError(
                 'missing required configuration parameter %s' % o
             )
@@ -94,18 +93,26 @@ class FilterSkeleton(ConfigurableBase):
 
         # Check the filter was sane
         if len(newmsg.headers()) < len(msg.headers()):
-            # Warn user
-            self.log.warning(
-                'Warning: filter %s returned fewer headers (%d) than supplied '
-                '(%d)\n'
-                % (self, len(newmsg.headers()), len(msg.headers()))
-            )
+            if not self.conf.get('ignore_header_shrinkage', False):
+                # Warn user
+                self.log.warning(
+                    'Warning: filter %s returned fewer headers (%d) than '
+                        'supplied (%d)\n'
+                    % (self, len(newmsg.headers()), len(msg.headers()))
+                )
 
         # Copy attributes from original message
         newmsg.copyattrs(msg)
 
         return newmsg
 
+    def some_security(self):
+        if (os.geteuid() == 0 and not self.conf['allow_root_commands']
+                and self.conf['user'] == None):
+            raise getmailConfigurationError(
+                'refuse to invoke external commands as root by default'
+            )
+
 #######################################
 class Filter_external(FilterSkeleton, ForkingBase):
     '''Arbitrary external filter destination.
@@ -174,6 +181,7 @@ class Filter_external(FilterSkeleton, ForkingBase):
         ConfString(name='user', required=False, default=None),
         ConfString(name='group', required=False, default=None),
         ConfBool(name='allow_root_commands', required=False, default=False),
+        ConfBool(name='ignore_header_shrinkage', required=False, default=False),
         ConfBool(name='ignore_stderr', required=False, default=False),
         ConfInstance(name='configparser', required=False),
     )
@@ -201,7 +209,7 @@ class Filter_external(FilterSkeleton, ForkingBase):
                 frozenset(self.exitcodes_drop)
             ):
                 raise getmailConfigurationError('exitcode sets intersect')
-        except ValueError, o:
+        except ValueError as o:
             raise getmailConfigurationError('invalid exit code specified (%s)'
                                             % o)
 
@@ -216,77 +224,42 @@ class Filter_external(FilterSkeleton, ForkingBase):
 
     def _filter_command(self, msg, msginfo, stdout, stderr):
         try:
-            # Write out message with native EOL convention
-            msgfile = tempfile.TemporaryFile()
-            msgfile.write(msg.flatten(False, False,
-                                      include_from=self.conf['unixfrom']))
-            msgfile.flush()
-            os.fsync(msgfile.fileno())
-            # Rewind
-            msgfile.seek(0)
-            # Set stdin to read from this file
-            os.dup2(msgfile.fileno(), 0)
-            # Set stdout and stderr to write to files
-            os.dup2(stdout.fileno(), 1)
-            os.dup2(stderr.fileno(), 2)
-            change_usergroup(None, self.conf['user'], self.conf['group'])
             args = [self.conf['path'], self.conf['path']]
             for arg in self.conf['arguments']:
                 arg = expand_user_vars(arg)
                 for (key, value) in msginfo.items():
                     arg = arg.replace('%%(%s)' % key, value)
                 args.append(arg)
-            # Can't log this; if --trace is on, it will be written to the
-            # message passed to the filter.
-            #self.log.debug('about to execl() with args %s\n' % str(args))
-            os.execl(*args)
-        except StandardError, o:
+            change_usergroup(None, self.conf['user'], self.conf['group'])
+            self.child_replace_me(msg, False, False, self.conf['unixfrom'],
+                                  stdout, stderr, args,
+                nolog=True) # ... else if --trace is on,
+                            # it will be written to the
+                            # message passed to the filter.
+        except Exception as o:
             # Child process; any error must cause us to exit nonzero for parent
             # to detect it
             self.log.critical('exec of filter %s failed (%s)'
                               % (self.conf['command'], o))
             os._exit(127)
 
-    def _filter_message(self, msg):
+    def _filter_message_common(self, msg):
         self.log.trace()
-        self._prepare_child()
-        msginfo = {}
-        msginfo['sender'] = msg.sender
-        if msg.recipient != None:
-            msginfo['recipient'] = msg.recipient
-            msginfo['domain'] = msg.recipient.lower().split('@')[-1]
-            msginfo['local'] = '@'.join(msg.recipient.split('@')[:-1])
-        self.log.debug('msginfo "%s"\n' % msginfo)
 
-        # At least some security...
-        if (os.geteuid() == 0 and not self.conf['allow_root_commands']
-                and self.conf['user'] == None):
-            raise getmailConfigurationError(
-                'refuse to invoke external commands as root by default'
-            )
+        msginfo = self.get_msginfo(msg)
 
-        stdout = tempfile.TemporaryFile()
-        stderr = tempfile.TemporaryFile()
-        childpid = os.fork()
+        self.some_security()
 
-        if not childpid:
-            # Child
-            self._filter_command(msg, msginfo, stdout, stderr)
-        self.log.debug('spawned child %d\n' % childpid)
+        return self.forkchild(
+            lambda o, e: self._filter_command(msg, msginfo, o, e)
+            , with_out=False)
 
-        # Parent
-        exitcode = self._wait_for_child(childpid)
-
-        stdout.seek(0)
-        stderr.seek(0)
-        err = stderr.read().strip()
-
-        self.log.debug('command %s %d exited %d\n' % (self.conf['command'],
-                                                      childpid, exitcode))
-
-        newmsg = Message(fromfile=stdout)
-
-        return (exitcode, newmsg, err)
+    def _filter_message(self, msg):
+        child = self._filter_message_common(msg)
+        self.log.debug('command %s %d exited %d\n' % (
+            self.conf['command'], child.childpid, child.exitcode))
+        newmsg = Message(fromfile=child.stdout)
+        return (child.exitcode, newmsg, child.err)
 
 #######################################
 class Filter_classifier(Filter_external):
@@ -305,52 +278,13 @@ class Filter_classifier(Filter_external):
         self.log.info('Filter_classifier(%s)\n' % self._confstring())
 
     def _filter_message(self, msg):
-        self.log.trace()
-        self._prepare_child()
-        msginfo = {}
-        msginfo['sender'] = msg.sender
-        if msg.recipient != None:
-            msginfo['recipient'] = msg.recipient
-            msginfo['domain'] = msg.recipient.lower().split('@')[-1]
-            msginfo['local'] = '@'.join(msg.recipient.split('@')[:-1])
-        self.log.debug('msginfo "%s"\n' % msginfo)
-
-        # At least some security...
-        if (os.geteuid() == 0 and not self.conf['allow_root_commands']
-                and self.conf['user'] == None):
-            raise getmailConfigurationError(
-                'refuse to invoke external commands as root by default'
-            )
-
-        stdout = tempfile.TemporaryFile()
-        stderr = tempfile.TemporaryFile()
-        childpid = os.fork()
-
-        if not childpid:
-            # Child
-            self._filter_command(msg, msginfo, stdout, stderr)
-        self.log.debug('spawned child %d\n' % childpid)
-
-        # Parent
-        exitcode = self._wait_for_child(childpid)
-
-        stdout.seek(0)
-        stderr.seek(0)
-        err = stderr.read().strip()
-
-        self.log.debug('command %s %d exited %d\n' % (self.conf['command'],
-                                                      childpid, exitcode))
-
-        for line in [line.strip() for line in stdout.readlines()
+        child = self._filter_message_common(msg)
+        self.log.debug('command %s %d exited %d\n' % (
+            self.conf['command'], child.childpid, child.exitcode))
+        for line in [line.strip() for line in child.stdout.readlines()
                      if line.strip()]:
-            # Output from filter can be in any random text encoding and may
-            # not even be valid, which causes problems when trying to stick
-            # that text into message headers.  Try to decode it to something
-            # sane here first.
-            line = decode_crappy_text(line)
             msg.add_header('X-getmail-filter-classifier', line)
-
-        return (exitcode, msg, err)
+        return (child.exitcode, msg, child.err)
 
 #######################################
 class Filter_TMDA(FilterSkeleton, ForkingBase):
@@ -411,33 +345,22 @@ class Filter_TMDA(FilterSkeleton, ForkingBase):
 
     def _filter_command(self, msg, stdout, stderr):
         try:
-            # Write out message with native EOL convention
-            msgfile = tempfile.TemporaryFile()
-            msgfile.write(msg.flatten(True, True, include_from=True))
-            msgfile.flush()
-            os.fsync(msgfile.fileno())
-            # Rewind
-            msgfile.seek(0)
-            # Set stdin to read from this file
-            os.dup2(msgfile.fileno(), 0)
-            # Set stdout and stderr to write to files
-            os.dup2(stdout.fileno(), 1)
-            os.dup2(stderr.fileno(), 2)
-            change_usergroup(None, self.conf['user'], self.conf['group'])
             args = [self.conf['path'], self.conf['path']]
             # Set environment for TMDA
-            os.environ['SENDER'] = msg.sender
-            os.environ['RECIPIENT'] = msg.recipient
+            os.environ['SENDER'] = msg.sender.strip()
+            rcpnt = msg.recipient.strip()
+            os.environ['RECIPIENT'] = rcpnt
             os.environ['EXT'] = self.conf['conf-break'].join(
-                '@'.join(msg.recipient.split('@')[:-1]).split(
+                '@'.join(rcpnt.split('@')[:-1]).split(
                     self.conf['conf-break']
                 )[1:]
             )
             self.log.trace('SENDER="%(SENDER)s",RECIPIENT="%(RECIPIENT)s"'
                            ',EXT="%(EXT)s"' % os.environ)
-            self.log.debug('about to execl() with args %s\n' % str(args))
-            os.execl(*args)
-        except StandardError, o:
+            change_usergroup(None, self.conf['user'], self.conf['group'])
+            self.child_replace_me(msg, True, True, True,
+                                  stdout, stderr, args)
+        except Exception as o:
             # Child process; any error must cause us to exit nonzero for parent
             # to detect it
             self.log.critical('exec of filter %s failed (%s)'
@@ -446,36 +369,17 @@ class Filter_TMDA(FilterSkeleton, ForkingBase):
 
     def _filter_message(self, msg):
         self.log.trace()
-        self._prepare_child()
         if msg.recipient == None or msg.sender == None:
             raise getmailConfigurationError(
                 'TMDA requires the message envelope and therefore a multidrop '
                 'retriever'
             )
+        self.some_security()
 
-        # At least some security...
-        if (os.geteuid() == 0 and not self.conf['allow_root_commands']
-                and self.conf['user'] == None):
-            raise getmailConfigurationError(
-                'refuse to invoke external commands as root by default'
-            )
+        child = self.forkchild(
+            lambda o,e: self._filter_command(msg, o, e)
+            , with_out=False)
 
-        stdout = tempfile.TemporaryFile()
-        stderr = tempfile.TemporaryFile()
-        childpid = os.fork()
-
-        if not childpid:
-            # Child
-            self._filter_command(msg, stdout, stderr)
-        self.log.debug('spawned child %d\n' % childpid)
-
-        # Parent
-        exitcode = self._wait_for_child(childpid)
-
-        stderr.seek(0)
-        err = stderr.read().strip()
-
-        self.log.debug('command %s %d exited %d\n' % (self.conf['command'],
-                                                      childpid, exitcode))
-
-        return (exitcode, msg, err)
+        self.log.debug('command %s %d exited %d\n' % (
+            self.conf['command'], child.childpid, child.exitcode))
+        return (child.exitcode, msg, child.err)
