@@ -1,4 +1,3 @@
-#!/usr/bin/env python2.3
 '''Logging support for getmail.
 
 The new standard Python libary module logging didn't cut it for me; it doesn't
@@ -52,6 +51,8 @@ class _Logger(object):
 
     def log(self, msglevel, msgtxt):
         '''Log a message of level <msglevel> containing text <msgtxt>.'''
+        if sys.version_info.major > 2 and isinstance(msgtxt,bytes):
+            msgtxt = msgtxt.decode()
         for handler in self.handlers:
             if msglevel < handler['minlevel'] or msglevel > handler['maxlevel']:
                 continue
