@@ -1,4 +1,3 @@
-#!/usr/bin/env python2.3
 '''The getmail Message class.
 
 '''
@@ -10,31 +9,26 @@ __all__ = [
 import sys
 import os
 import time
-import cStringIO
 import re
 import email
-import email.Errors
-import email.Utils
-import email.Parser
-from email.Generator import Generator
-try:
+try: #py2
+    import email.Errors as Errors
+    import email.Utils as Utils
+    import email.Parser as Parser
+    from email.Generator import Generator
+    from email.Header import Header
+except ImportError:
+    import email.errors as Errors
+    import email.utils as Utils
+    import email.parser as Parser
+    from email.generator import Generator
     from email.header import Header
-except ImportError, o:
-    # Python < 2.5
-    from email import Header
 
 from getmailcore.exceptions import *
 from getmailcore.utilities import mbox_from_escape, format_header, \
     address_no_brackets
 import getmailcore.logging
 
-if sys.hexversion < 0x02040000:
-    # email module in Python 2.3 uses more recursion to parse messages or
-    # similar; a user reported recursion errors with a message with ~300
-    # MIME parts.
-    # Hack around it by increasing the recursion limit.
-    sys.setrecursionlimit(2000)
-
 message_attributes = (
     'sender',
     'received_by',
@@ -43,8 +37,7 @@ message_attributes = (
     'recipient'
 )
 
-RE_FROMLINE = re.compile(r'^(>*From )', re.MULTILINE)
-
+_NL = os.linesep.encode()
 
 #######################################
 def corrupt_message(why, fromlines=None, fromstring=None):
@@ -56,22 +49,22 @@ def corrupt_message(why, fromlines=None, fromstring=None):
     msg = email.message_from_string('')
     msg['From'] = '"unknown sender" <>'
     msg['Subject'] = 'Corrupt message received'
-    msg['Date'] = email.Utils.formatdate(localtime=True)
+    msg['Date'] = Utils.formatdate(localtime=True)
     body = [
-        'A badly-corrupt message was retrieved and could not be parsed',
-        'for the following reason:',
-        '',
-        '    %s' % why,
-        '',
-        'Below the following line is the original message contents.',
-        '',
-        '--------------------------------------------------------------',
+        b'A badly-corrupt message was retrieved and could not be parsed',
+        b'for the following reason:',
+        b'',
+        b'    %s' % why,
+        b'',
+        b'Below the following line is the original message contents.',
+        b'',
+        b'--------------------------------------------------------------',
     ]
     if fromlines:
         body.extend([line.rstrip() for line in fromlines])
     elif fromstring:
         body.extend([line.rstrip() for line in fromstring.splitlines()])
-    msg.set_payload(os.linesep.join(body))
+    msg.set_payload(_NL.join(body))
     for attr in message_attributes:
         setattr(msg, attr, '')
     return msg
@@ -99,7 +92,13 @@ class Message(object):
         self.received_from = None
         self.received_with = None
         self.__raw = None
-        parser = email.Parser.Parser()
+        try:
+            parser = Parser.BytesParser()
+            parsestr = parser.parsebytes
+        except:
+            parser = Parser.Parser()
+            parsestr = parser.parsestr
+
 
         # Message is instantiated with fromlines for POP3, fromstring for
         # IMAP (both of which can be badly-corrupted or invalid, i.e. spam,
@@ -107,20 +106,20 @@ class Message(object):
         # of filters, etc, which should be saner.
         if fromlines:
             try:
-                self.__msg = parser.parsestr(os.linesep.join(fromlines))
-            except email.Errors.MessageError, o:
+                self.__msg = parsestr(_NL.join(fromlines))
+            except Errors.MessageError as o:
                 self.__msg = corrupt_message(o, fromlines=fromlines)
-            self.__raw = os.linesep.join(fromlines)
+            self.__raw = _NL.join(fromlines)
         elif fromstring:
             try:
-                self.__msg = parser.parsestr(fromstring)
-            except email.Errors.MessageError, o:
+                self.__msg = parsestr(fromstring)
+            except Errors.MessageError as o:
                 self.__msg = corrupt_message(o, fromstring=fromstring)
             self.__raw = fromstring
         elif fromfile:
             try:
                 self.__msg = parser.parse(fromfile)
-            except email.Errors.MessageError, o:
+            except Errors.MessageError as o:
                 # Shouldn't happen
                 self.__msg = corrupt_message(o, fromstring=fromfile.read())
             # fromfile is only used by getmail_maildir, getmail_mbox, and
@@ -147,8 +146,8 @@ class Message(object):
         it by writing out what we need, letting the generator write out the
         message, splitting it into lines, and joining them with the platform
         EOL.
-        
-        Note on mangle_from: the Python email.Generator class apparently only
+
+        Note on mangle_from: the Python Generator class apparently only
         quotes "From ", not ">From " (i.e. it uses mboxo format instead of
         mboxrd).  So we don't use its mangling, and do it by hand instead.
         '''
@@ -167,28 +166,31 @@ class Message(object):
         else:
             dtline = ''
         if received:
-            content = 'from %s by %s with %s' % (
+            rcvd = 'from %s by %s with %s' % (
                 self.received_from, self.received_by, self.received_with
             )
             if self.recipient is not None:
-                content += ' for <%s>' % self.recipient
-            content += '; ' + time.strftime('%d %b %Y %H:%M:%S -0000',
+                rcvd += ' for <%s>' % self.recipient
+            rcvd += '; ' + time.strftime('%d %b %Y %H:%M:%S -0000',
                                             time.gmtime())
-            receivedline = format_header('Received', content)
+            rcvline = format_header('Received', rcvd)
         else:
-            receivedline = ''
+            rcvline = ''
         # From_ handled above, always tell the generator not to include it
         try:
-            tmpf = cStringIO.StringIO()
-            gen = Generator(tmpf, False, 0)
-            gen.flatten(self.__msg, False)
-            strmsg = tmpf.getvalue()
+            try:
+                strmsg = self.__msg.as_bytes(
+                    policy=self.__msg.policy.clone(linesep=os.linesep))
+            except AttributeError:
+                strmsg = self.__msg.as_string()
+                strmsg = _NL.join(strmsg.splitlines() + [b''])
             if mangle_from:
-                # do mboxrd-style "From " line quoting
-                strmsg = RE_FROMLINE.sub(r'>\1', strmsg)
-            return (fromline + rpline + dtline + receivedline 
-                    + os.linesep.join(strmsg.splitlines() + ['']))
-        except TypeError, o:
+                # do mboxrd-style "From " line quoting (add one '>')
+                RE_FROMLINE = re.compile(b'^(>*From )', re.MULTILINE)
+                strmsg = RE_FROMLINE.sub(b'>\\1', strmsg)
+
+            return ((fromline+rpline+dtline+rcvline).encode()+strmsg)
+        except TypeError as o:
             # email module chokes on some badly-misformatted messages, even
             # late during flatten().  Hope this is fixed in Python 2.4.
             if self.__raw is None:
@@ -201,7 +203,7 @@ class Message(object):
                                 include_from)
 
     def add_header(self, name, content):
-        self.__msg[name] = Header(content.rstrip())
+        self.__msg[name] = Header(content.rstrip(), 'utf-8')
 
     def remove_header(self, name):
         del self.__msg[name]
