@@ -1,4 +1,3 @@
-#!/usr/bin/env python2.3
 '''Classes implementing retrievers (message sources getmail can retrieve mail
 from).
 
@@ -93,7 +92,7 @@ class SimplePOP3SSLRetriever(POP3RetrieverBase, POP3SSLinitMixIn):
 
         ConfInt(name='timeout', required=False, default=180),
         ConfString(name='server'),
-        ConfInt(name='port', required=False, default=POP3_ssl_port),
+        ConfInt(name='port', required=False, default=POP3_SSL_PORT),
         ConfString(name='username'),
         ConfPassword(name='password', required=False, default=None),
         ConfTupleOfStrings(name='password_command', required=False, default=()),
@@ -157,7 +156,7 @@ class BrokenUIDLPOP3RetrieverBase(POP3RetrieverBase):
                 self.msgid_by_msgnum[msgnum] = msgnum
                 self.msgsizes[msgnum] = msgsize
             self.sorted_msgnum_msgid = sorted(self.msgid_by_msgnum.items())
-        except poplib.error_proto, o:
+        except poplib.error_proto as o:
             raise getmailOperationError('POP error (%s)' % o)
         self.gotmsglist = True
 
@@ -202,7 +201,7 @@ class BrokenUIDLPOP3SSLRetriever(BrokenUIDLPOP3RetrieverBase, POP3SSLinitMixIn):
 
         ConfInt(name='timeout', required=False, default=180),
         ConfString(name='server'),
-        ConfInt(name='port', required=False, default=POP3_ssl_port),
+        ConfInt(name='port', required=False, default=POP3_SSL_PORT),
         ConfString(name='username'),
         ConfPassword(name='password', required=False, default=None),
         ConfTupleOfStrings(name='password_command', required=False, default=()),
@@ -274,7 +273,7 @@ class MultidropPOP3SSLRetriever(MultidropPOP3RetrieverBase, POP3SSLinitMixIn):
 
         ConfInt(name='timeout', required=False, default=180),
         ConfString(name='server'),
-        ConfInt(name='port', required=False, default=POP3_ssl_port),
+        ConfInt(name='port', required=False, default=POP3_SSL_PORT),
         ConfString(name='username'),
         ConfPassword(name='password', required=False, default=None),
         ConfTupleOfStrings(name='password_command', required=False, default=()),
@@ -355,7 +354,7 @@ class MultidropSDPSRetriever(SimplePOP3Retriever, POP3initMixIn):
         try:
             msgnum = self._getmsgnumbyid(msgid)
             resp, lines, octets = self.conn._longcmd('*ENV %i' % msgnum)
-        except poplib.error_proto, o:
+        except poplib.error_proto as o:
             raise getmailConfigurationError(
                 'server does not support *ENV (%s)' % o
             )
@@ -388,6 +387,7 @@ class SimpleIMAPRetriever(IMAPRetrieverBase, IMAPinitMixIn):
         # .authenticate(), so we can't do this yet (?).
         ConfBool(name='use_cram_md5', required=False, default=False),
         ConfBool(name='use_kerberos', required=False, default=False),
+        ConfBool(name='use_xoauth2', required=False, default=False),
     )
     received_from = None
     received_with = 'IMAP4'
@@ -437,6 +437,7 @@ class SimpleIMAPSSLRetriever(IMAPRetrieverBase, IMAPSSLinitMixIn):
         # .authenticate(), so we can't do this yet (?).
         ConfBool(name='use_cram_md5', required=False, default=False),
         ConfBool(name='use_kerberos', required=False, default=False),
+        ConfBool(name='use_xoauth2', required=False, default=False),
         ConfString(name='ssl_cert_hostname', required=False, default=None),
     )
     received_from = None
@@ -479,6 +480,7 @@ class MultidropIMAPRetriever(MultidropIMAPRetrieverBase, IMAPinitMixIn):
         # .authenticate(), so we can't do this yet (?).
         ConfBool(name='use_cram_md5', required=False, default=False),
         ConfBool(name='use_kerberos', required=False, default=False),
+        ConfBool(name='use_xoauth2', required=False, default=False),
         ConfString(name='envelope_recipient'),
     )
     received_from = None
@@ -529,6 +531,7 @@ class MultidropIMAPSSLRetriever(MultidropIMAPRetrieverBase, IMAPSSLinitMixIn):
         # .authenticate(), so we can't do this yet (?).
         ConfBool(name='use_cram_md5', required=False, default=False),
         ConfBool(name='use_kerberos', required=False, default=False),
+        ConfBool(name='use_xoauth2', required=False, default=False),
         ConfString(name='envelope_recipient'),
         ConfString(name='ssl_cert_hostname', required=False, default=None),
     )
